{% extends "wagtailadmin/generic/base.html" %}
{% load i18n %}

{% block main_header %}
    {% if not breadcrumbs_items %}
        {% include "wagtailadmin/shared/header.html" with title=page_title subtitle=page_subtitle action_url=header_action_url action_text=header_action_label icon=header_icon only %}
    {% endif %}
{% endblock %}

{% block content %}
    {% block header %}
        {{ block.super }}
    {% endblock %}

    {% block listing %}
        <div class="{% if filters and not breadcrumbs_items %}filterable{% endif %}">
            <div id="listing-results">
                {% comment %}
                    This div will be replaced on AJAX refreshes.
                    Do not add page furniture here unless you intend it to disappear on AJAX refresh
                {% endcomment %}
                {% include view.results_template_name|default:"wagtailadmin/generic/listing_results.html" %}
            </div>
            {% if filters and not breadcrumbs_items %}
                {% include "wagtailadmin/shared/filters.html" %}
            {% endif %}
        </div>
    {% endblock %}

    {% block bulk_actions %}{% endblock %}
{% endblock %}

{% block extra_js %}
    {{ block.super }}
    {{ media.js }}
{% endblock %}

{% block extra_css %}
    {{ block.super }}
    {{ media.css }}
{% endblock %}
