{% extends "wagtailadmin/tables/title_cell.html" %}
{% load i18n %}

{% block title %}
    <a href="{{ link_url }}">
        {% blocktrans trimmed count counter=instance.all_pages.count %}
            1 page
        {% plural %}
            {{ counter }} pages
        {% endblocktrans %}
    </a>
{% endblock %}

{% block extra %}
    {# Only show snippet type count if there's a model with workflows enabled #}
    {% if workflow_enabled_models %}
        <div>
            {% blocktrans trimmed count counter=instance.content_types %}
                1 snippet type
            {% plural %}
                {{ counter }} snippet types
            {% endblocktrans %}
        </div>
    {% endif %}
{% endblock %}
