{% extends "wagtailadmin/generic/create.html" %}
{% load i18n wagtailadmin_tags %}

{% block slim_header %}
    {% comment %}
        We can't use the `only` keyword yet, because the SnippetStatusSidePanel still rely on some
        context variables from the view. As a result, we can't reuse the include from the generic
        template and have to copy it here without the `only` keyword.
    {% endcomment %}
    {% include 'wagtailadmin/shared/headers/slim_header.html' with breadcrumbs_items=breadcrumbs_items side_panels=side_panels history_url=history_url buttons=header_buttons %}
{% endblock %}

{% block actions %}
    <footer class="footer">
        <nav class="actions actions--primary footer__container" aria-label="{% trans 'Actions' %}">
            {{ action_menu.render_html }}
        </nav>
    </footer>
{% endblock %}

{% block extra_js %}
    {{ block.super }}

    <script>
        // Set wagtailConfig.ACTIVE_CONTENT_LOCALE if this is a translated page
        // This value may be used by JS widgets on the form
        {% if locale %}
            window.wagtailConfig = window.wagtailConfig || {};
            window.wagtailConfig.ACTIVE_CONTENT_LOCALE = '{{ locale.language_code|escapejs }}';
        {% endif %}
    </script>
{% endblock %}
